import { BrowserNativeObject, ErrorOption, FieldErrors, FieldNamesMarkedBoolean, FieldPath, FieldPathValues, FieldValues, GlobalError, InternalFieldName, Message, Primitive, SetFocusOptions, UseControllerProps, UseControllerReturn, UseFieldArrayReturn, UseFormProps, UseFormRegisterReturn, UseFormReturn, UseFormSetError, UseFormStateProps, UseWatchProps, ValidateResult, ValidationRule } from "react-hook-form";
/**
 * Represents a field or collection of fields.
 *
 * Can be called which is equivalent to calling RHF's `register()` with the corresponding field name.
 *
 * Use `FormBuilder<T>` as the expected prop to build typed form field
 * components (e.g. `FormBuilder<number>` for numeric inputs).
 */
export type FormBuilder<T> = FormBuilderRegisterFn<T> & {
    $useController<U = T>(props?: Omit<UseControllerProps, "name" | "control">): UseControllerReturn<{
        __: U;
    }, U extends Primitive | BrowserNativeObject ? "__" : "__">;
    $useWatch<U = T>(props?: $UseWatchCommonProps): U;
    $useWatch<TValues extends FieldValues = T extends FieldValues ? T : FieldValues, TFieldNames extends readonly FieldPath<TValues>[] = readonly FieldPath<TValues>[]>(props: $UseWatchCommonProps & {
        name: readonly [...TFieldNames];
    }): FieldPathValues<TValues, TFieldNames>;
    $useState<U = T>(props?: $UseStateProps): U extends FieldValues ? {
        errors: FieldErrors<U>;
        dirty: Partial<Readonly<FieldNamesMarkedBoolean<U>>>;
    } : {
        errors: GlobalError;
        dirty: boolean;
    };
    $setValue(value: T): void;
    $setError(error: ErrorOption, options: Parameters<UseFormSetError<FieldValues>>[2]): void;
    $setFocus(options: SetFocusOptions): void;
    $discriminate<TKey extends keyof T, TValue extends T[TKey]>(key: TKey, value: TValue): IsUnknown<T> extends 1 ? FormBuilder<unknown> : FormBuilder<Extract<T, Record<TKey, TValue>>>;
} & (T extends Primitive ? unknown : T extends Array<infer U extends FieldValues> ? {
    [K: number]: FormBuilder<U>;
    $useFieldArray<TItem = U>(props?: $UseFieldArrayProps<U>): $UseFieldArrayReturn<TItem>;
} : {
    [K in Exclude<keyof T, `${"__"}${string}`>]-?: FormBuilder<T[K] extends Primitive ? T[K] : NonNullable<T[K]>>;
});
type FormBuilderRegisterFn<T> = {
    bivarianceHack(options?: RegisterOptions<T>): UseFormRegisterReturn;
}["bivarianceHack"];
export declare function createFormBuilder<TFieldValues extends FieldValues>(methods: UseFormReturn<TFieldValues>, path: string[], key?: string): FormBuilder<TFieldValues>;
export interface UseFormBuilderReturn<TFieldValues extends FieldValues, TContext extends object = object> extends UseFormReturn<TFieldValues, TContext> {
    fields: FormBuilder<TFieldValues>;
}
/**
 * A type-safe alternative to `useForm()`.
 *
 * @example ```tsx
 * const {fields, ...methods} = useFormBuilder({...});
 *
 * // Spread result the same way as with `register`.
 * <input type='number' {...fields.foo.bar({min: ..., max: ...})} />
 *
 * // Field array (only available on array values, name is pre-filled)
 * const {fields, ...arrayMethods} = fields.things.$useFieldArray({...});
 * ```
 * @param props additional props passed to `useForm()`
 * @see FormBuilder
 */
export declare function useFormBuilder<TFieldValues extends FieldValues = FieldValues, TContext extends object = object>(props?: UseFormBuilderProps<TFieldValues, TContext>): UseFormBuilderReturn<TFieldValues, TContext>;
type Validate<T> = {
    bivarianceHack(value: T): ValidateResult;
}["bivarianceHack"];
export type RegisterOptions<T> = Partial<{
    required: Message | ValidationRule<boolean>;
    min: ValidationRule<number | string>;
    max: ValidationRule<number | string>;
    maxLength: ValidationRule<number>;
    minLength: ValidationRule<number>;
    validate: Validate<T> | Record<string, Validate<T>>;
    value: T;
    setValueAs: (value: unknown) => T;
    shouldUnregister?: boolean;
    onChange?: (event: Event) => void;
    onBlur?: (event: Event) => void;
    disabled: boolean;
    deps: InternalFieldName | InternalFieldName[];
}> & ({
    pattern?: ValidationRule<RegExp>;
    valueAsNumber?: false;
    valueAsDate?: false;
} | {
    pattern?: undefined;
    valueAsNumber?: false;
    valueAsDate?: true;
} | {
    pattern?: undefined;
    valueAsNumber?: true;
    valueAsDate?: false;
});
type $UseWatchCommonProps = Omit<UseWatchProps<never>, "name" | "control" | "defaultValue">;
type $UseStateProps = Omit<UseFormStateProps<never>, "name" | "control">;
interface $UseFieldArrayProps<T> {
    rules?: {} & Pick<RegisterOptions<T>, "maxLength" | "minLength" | "required">;
    shouldUnregister?: boolean;
}
type $UseFieldArrayReturn<T> = Omit<UseFieldArrayReturn<{
    __: T[];
}, T extends Primitive | BrowserNativeObject ? never : "__">, "fields"> & {
    fields: (FormBuilder<T> & {
        $key: string;
    })[];
};
export type UseFormBuilderProps<TFieldValues extends FieldValues = FieldValues, TContext = any> = UseFormProps<TFieldValues, TContext>;
type IsUnknown<T> = unknown extends T ? (T extends unknown ? 1 : 0) : 0;
export {};
